#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "titles.h"
#include "moves.h"

#define MAX_PLAYERS 4
#define NO_SCORE -1
#define ADJ_SCORE(a) ((a)==NO_SCORE ? 0 : (a))

int get_num_players();
void print_scores();
int want_to_play_again();
int top_bonus(int scores[NTITLES]);
int top_total(int scores[NTITLES]);
int bot_total(int scores[NTITLES]);
int grand_total(int scores[NTITLES]);

int main() {
  srandom(time(0));
  do {
    int move, player, nplayers;
    int score[MAX_PLAYERS][NTITLES];

    /* initialize all of the scores to NO_SCORE (not yet scored) */
    for (player=0; player<MAX_PLAYERS; player++) {
      for (move=0; move<NMOVES; move++)
        score[player][move] = NO_SCORE;
      score[player][YAHT_BONUS] = 0;  /* yahtzee bonus */
    }

    nplayers = get_num_players();

    for (move=0; move<NMOVES; move++)
      for (player=0; player<nplayers; player++) {
        print_scores(nplayers, score);
        do_move(player, score[player]);
      }
    printf("\n\n=== Final scores ===\n");
    print_scores(nplayers, score);
  } while (want_to_play_again());
  return 0;
}

int get_num_players() {
  return 1;
}

void print_scores(int nplayers, int score[MAX_PLAYERS][NTITLES]) {
  int i, j;
  printf(SPC_STRING, " ");
  for (i=0; i<nplayers; i++)
    printf("| player %3d ", i+1);
  putchar('\n');

  /* ones through sixes */
  for (i=ONES; i<=SIXES; i++) {
    printf(SPC_STRING, titles[i]);
    for (j=0; j<nplayers; j++)
      if (score[j][i] == NO_SCORE)
        printf("|          - ");
      else
        printf("|        %3d ", score[j][i]);
    putchar('\n');
  }

  /* top bonus */
  printf(SPC_STRING, bonus_titles[TOP_BONUS]);
  for (j=0; j<nplayers; j++)
    printf("|        %3d ", top_bonus(score[j]));
  putchar('\n');

  /* top total */
  printf(SPC_STRING, bonus_titles[TOP_TOTAL]);
  for (j=0; j<nplayers; j++)
    printf("|        %3d ", top_total(score[j]));
  putchar('\n');

  /* 3-of-a-kind through chance */
  for (i=THREE_KIND; i<=CHANCE; i++) {
    printf(SPC_STRING, titles[i]);
    for (j=0; j<nplayers; j++)
      if (score[j][i] == NO_SCORE)
        printf("|          - ");
      else
        printf("|        %3d ", score[j][i]);
    putchar('\n');
  }

  /* yahtzee bonus */
  printf(SPC_STRING, bonus_titles[YAHTZEE_BONUS]);
  for (j=0; j<nplayers; j++)
    printf("|        %3d ", score[j][YAHT_BONUS]);
  putchar('\n');

  /* bottom total */
  printf(SPC_STRING, bonus_titles[BOT_TOTAL]);
  for (j=0; j<nplayers; j++)
    printf("|        %3d ", bot_total(score[j]));
  putchar('\n');

  /* grand total */
  printf(SPC_STRING, bonus_titles[GR_TOTAL]);
  for (j=0; j<nplayers; j++)
    printf("|        %3d ", grand_total(score[j]));
  putchar('\n');
}

int want_to_play_again() {
  return 0;
}

int top_bonus(int scores[NTITLES]) {
  int i, total = 0;
  for (i=ONES; i<=SIXES; i++)
    total += ADJ_SCORE(scores[i]);
  return total>=63 ? 35 : 0;
}

int top_total(int scores[NTITLES]) {
  int i, total = 0;
  for (i=ONES; i<=SIXES; i++)
    total += ADJ_SCORE(scores[i]);
  return total>=63 ? total+35 : total;
}

int bot_total(int scores[NTITLES]) {
  int i, total = 0;
  for (i=THREE_KIND; i<=YAHT_BONUS; i++)
    total += ADJ_SCORE(scores[i]);
  return total;
}

int grand_total(int scores[NTITLES]) {
  return top_total(scores) + bot_total(scores);
}
